﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace CoolKidEngine.Display
{
	public class Sprite : DisplayObject
	{
		public Vector2 Position { get; set; }
		public Rectangle SourceRectangle { get; set; }
		public Color Color { get; set; }

		public Sprite(Texture2D texture, Rectangle source, Color color)
			: base(texture)
		{
			SourceRectangle = new Rectangle(source.X, source.Y, source.Width, source.Height);
			Color = new Color(color.R, color.G, color.B, color.A);
			IsActive = true;
		}

		public override void Update(GameTime time)
		{
			
		}

		protected virtual void DrawTexture(SpriteBatch sb)
		{
			sb.Draw(Texture, Position, SourceRectangle, Color);
		}

		public override void Draw(SpriteBatch sb, GameTime time)
		{
			if (IsActive)
				DrawTexture(sb);
		}

		protected override object CloneInternal(object obj)
		{
			Sprite sprite = (Sprite)base.CloneInternal(obj);
			sprite.Position = new Vector2(Position.X, Position.Y);
			return sprite;
		}

		public override object Clone()
		{
			return CloneInternal(new Sprite(Texture, SourceRectangle, Color));
		}
	}
}
