﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace CoolKidEngine.Display
{
	public class Tile : DisplayObject
	{
		public Rectangle DestRectangle { get; set; }
		public Color Color { get; set; }

		/// <summary>
		/// Will get or set the sprite rect's center location
		/// </summary>
		public Vector2 Center
		{
			get
			{
				return new Vector2(DestRectangle.X + ((DestRectangle.Width / 2) * 1), DestRectangle.Y + ((DestRectangle.Height / 2) * 1));
			}
			set
			{
				Position = new Vector2(value.X - ((DestRectangle.Width / 2) * 1), value.Y - ((DestRectangle.Height / 2) * 1));
			}
		}

		public Vector2 Position
		{
			get { return new Vector2(DestRectangle.X, DestRectangle.Y); }
			set { DestRectangle = new Rectangle((int)value.X, (int)value.Y, DestRectangle.Width, DestRectangle.Height); }
		}

		public Tile(Texture2D texture, Rectangle destRect, Color color)
			:base(texture)
		{
			DestRectangle = new Rectangle(destRect.X, destRect.Y, destRect.Width, destRect.Height);
			Color = new Color(color.R, color.G, color.B, color.A);
			IsActive = true;
		}

		public override void Update(GameTime time)
		{
			
		}

		protected virtual void DrawTexture(SpriteBatch sb)
		{
			sb.Draw(Texture, DestRectangle, Color);
		}

		public override void Draw(SpriteBatch sb, GameTime time)
		{
			if (IsActive)
				DrawTexture(sb);
		}

		protected override object CloneInternal(object obj)
		{
			Tile tile = (Tile)base.CloneInternal(obj);
			return tile;
		}

		public override object Clone()
		{
			return CloneInternal(new Tile(Texture, DestRectangle, Color));
		}
	}
}
