﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using CoolKidEngine.Display;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace CoolKidEngine.Menu
{
	public class Label : Tile
	{
		protected SpriteFont Font = null;
		public string Text = null;
		protected Color FontColor { get; set; }

		Vector2 TextSize
		{
			get
			{
				if (Font == null)
					return Vector2.Zero;
				if (Text.Length <= 0)
					return Vector2.Zero;

				Text = Text.Replace("\t", "    ");
				return Font.MeasureString(Text);
			}
		}

		Vector2 TextPosition
		{
			get
			{
				return new Vector2
				(
					(DestRectangle.Width / 2 - TextSize.Length() / 2)
						+ this.DestRectangle.X,
					(DestRectangle.Height / 4) + this.DestRectangle.Y 
				);
			}
		}

		public Label(Texture2D texture, Rectangle destRect, Color color, SpriteFont font, Color fontColor, string text)
			:base(texture, destRect, color)
		{
			Font = font;
			Text = text;
			FontColor = new Color(fontColor.R, fontColor.G, fontColor.B, fontColor.A);
			IsActive = true;
		}

		protected override void DrawTexture(SpriteBatch sb)
		{
			base.DrawTexture(sb);

			sb.DrawString(Font, Text, TextPosition, FontColor);
		}

		protected override object CloneInternal(object obj)
		{
			return base.CloneInternal(obj);
		}

		public override object Clone()
		{
			return CloneInternal(new Label(Texture, DestRectangle, Color, Font, FontColor, Text));
		}
	}
}
