﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using CoolKidEngine.Utility;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace CoolKidEngine.Menu
{
	public class Menu
	{
		List<Button> ButtonList = new List<Button>();
		List<Label> LabelList = new List<Label>();

		Button BaseButton = null;
		public Label BaseLabel = null; //Want to change color on the fly, so made it public

		public Menu(Button baseButton, Label baseLabel)
		{
			BaseButton = (Button)baseButton.Clone();
			BaseLabel = (Label)baseLabel.Clone();
		}

		public void AddButton(string text, Vector2 postion, Action buttonAction)
		{
			Button newButton = (Button)BaseButton.Clone();
			newButton.Text = text;
			newButton.Center = postion;
			newButton.SetButtonAction(buttonAction);
			ButtonList.Add(newButton);
		}

		public void AddButton(string text, Rectangle rect, Action buttonAction)
		{
			Button newButton = (Button)BaseButton.Clone();
			newButton.Text = text;
			newButton.DestRectangle = rect;
			newButton.SetButtonAction(buttonAction);
			ButtonList.Add(newButton);
		}

		public void AddLabel(string text, Vector2 postion)
		{
			Label newLabel = (Label)BaseLabel.Clone();
			newLabel.Text = text;
			newLabel.Center = postion;
			LabelList.Add(newLabel);
		}

		public void AddPanel(Vector2 center, Vector2 size, Color color)
		{
			Label newLabel = (Label)BaseLabel.Clone();
			newLabel.DestRectangle = new Rectangle(0, 0, (int)size.X, (int)size.Y);
			newLabel.Center = center;
			newLabel.Text = "";
			newLabel.Color = color;
			LabelList.Add(newLabel);
		}

		public void AlignButtonsAndLabelsAsDropdown(Vector2 pos, bool leftAlign = true)
		{
			Vector2 offset = new Vector2(0, 0);
			
			foreach(var label in LabelList)
			{
				if (!leftAlign)
					offset.X = -label.DestRectangle.Width;
                label.Position = pos + offset;
				offset += new Vector2(0, label.DestRectangle.Height);
			}
			foreach (var button in ButtonList)
			{
				if (!leftAlign)
					offset.X = -button.DestRectangle.Width;
				button.Position = pos + offset;
				offset += new Vector2(0, button.DestRectangle.Height);
			}

		}

		public void ClickButtons(MouseState pState, MouseState cState, Vector2? mouseCamPos = null)
		{
			if (pState.LeftButton == ButtonState.Pressed
				&& cState.LeftButton == ButtonState.Released)
			{
				foreach (var button in ButtonList.Where(x => x.IsActive)
					.ToArray()) //Because the action might change the list
				{
					if (mouseCamPos.HasValue)
						button.ClickButton(new Point((int)mouseCamPos.Value.X, (int)mouseCamPos.Value.Y));
					else
						button.ClickButton(cState.Position);
				}
			}
		}

		public bool LeftClickLabels(MouseState pState, MouseState cState, Vector2? mouseCamPos = null)
		{
			bool didClickOnLabel = false;
			if (pState.LeftButton == ButtonState.Pressed
				&& cState.LeftButton == ButtonState.Released)
			{
				didClickOnLabel = ClickLabels(cState, mouseCamPos);

				if (didClickOnLabel)
					return didClickOnLabel;
			}
			return didClickOnLabel;
        }

		public bool RightClickLabels(MouseState pState, MouseState cState, Vector2? mouseCamPos = null)
		{
			bool didClickOnLabel = false;
			if (pState.RightButton == ButtonState.Pressed
				&& cState.RightButton == ButtonState.Released)
			{
				didClickOnLabel = ClickLabels(cState, mouseCamPos);

				if (didClickOnLabel)
					return didClickOnLabel;
			}
			return didClickOnLabel;
		}

		private bool ClickLabels(MouseState cState, Vector2? mouseCamPos)
		{
			bool didClickOnLabel = false;
			foreach (var label in LabelList.Where(x => x.IsActive)
					.ToArray()) //Because the action might change the list
			{
				if (mouseCamPos.HasValue)
					didClickOnLabel = CollisionHelper.RectToPoint(label.DestRectangle, new Point((int)mouseCamPos.Value.X, (int)mouseCamPos.Value.Y));
				else
					didClickOnLabel = CollisionHelper.RectToPoint(label.DestRectangle, cState.Position);

				if (didClickOnLabel)
					return didClickOnLabel;
			}
			return didClickOnLabel;
		}

		public virtual void ClearMenu()
		{
			LabelList.Clear();
			ButtonList.Clear();
		}

		public virtual void Draw(SpriteBatch sb, GameTime time)
		{
			foreach(var label in LabelList)
			{
				label.Draw(sb, time);
			}

			foreach(var button in ButtonList)
			{
				button.Draw(sb, time);
			}
		}

		public virtual object CloneInternal(object obj)
		{
			var menuObj = (Menu)obj;
			//decided to just not pass the current buttons and labels
			return menuObj;
		}

		public virtual object Clone()
		{
			return CloneInternal(new Menu(BaseButton, BaseLabel));
		}
	}
}
