﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace RitualisticCapitalism
{
	public class Camera2D
	{
		private float _zoomUpperLimit;
		private float _zoomLowerLimit;

		private float _zoom;
		private Matrix _transform;
		private Vector2 _pos;
		private float _rotation;
		private int _viewportWidth;
		private int _viewportHeight;
		//private int _worldWidth;
		//private int _worldHeight;
		private float _panSpeed;
		private float _zoomSpeed;

		Rectangle? cameraBoundry = null;

		public Camera2D(Viewport viewport, Vector2 pos, float initialZoom, float zoomUpperLimit, float zoomLowerLimit, float panSpeed = 8, float zoomSpeed = 0.1f)
		{
			_zoomLowerLimit = zoomLowerLimit;
			_zoomUpperLimit = zoomUpperLimit;
			_panSpeed = panSpeed;
			_zoomSpeed = zoomSpeed;
			_zoom = initialZoom;
			_rotation = 0.0f;
			_pos = pos; //* 1.5f/initialZoom;
						//_pos = new Vector2(0, 0);
			_viewportWidth = viewport.Width;
			_viewportHeight = viewport.Height;
			//_worldWidth = worldWidth;
			//_worldHeight = worldHeight;
		}

		public void SetCameraBoundry(Rectangle inRect)
		{
			cameraBoundry = inRect;
		}

		#region Properties

		public float Zoom
		{
			get { return _zoom; }
			set
			{
				_zoom = value;
				if (_zoom < _zoomLowerLimit)
					_zoom = _zoomLowerLimit;
				if (_zoom > _zoomUpperLimit)
					_zoom = _zoomUpperLimit;
			}
		}

		public float Rotation
		{
			get { return _rotation; }
			set { _rotation = value; }
		}

		public void Move(Vector2 amount)
		{
			_pos += amount;
		}

		public Vector2 Pos
		{
			get { return _pos; }
			set
			{
				if (cameraBoundry.HasValue)
				{
					Vector2 nextPos = value;

					if (cameraBoundry.Value.Intersects(new Rectangle((int)nextPos.X, (int)_pos.Y, 1, 1)))
						_pos = new Vector2(nextPos.X, _pos.Y);
					if (cameraBoundry.Value.Intersects(new Rectangle((int)_pos.X, (int)nextPos.Y, 1, 1)))
						_pos = new Vector2(_pos.X, nextPos.Y);
				}
				else
					_pos = value;
			}
		}

		#endregion

		public bool IsWithinUpperLimit(float distance)
		{
			if (Math.Abs(Zoom - _zoomUpperLimit) < distance)
				return true;
			else
				return false;
		}

		public bool IsWithinLowerLimit(float distance)
		{
			if (Math.Abs(Zoom - _zoomLowerLimit) < distance)
				return true;
			else
				return false;
		}

		public void SetZoomUpperLimit(float distance)
		{
			Zoom = _zoomUpperLimit;
			Zoom -= distance;
		}

		public void SetZoomLowerLimit(float distance)
		{
			Zoom = _zoomLowerLimit;
			Zoom += distance;
		}

		public void SetZoom(float inAmount)
		{
			Zoom = inAmount;
		}

		//-------------------------------
		// Example Draw Code for using the Camera
		//-------------------------------
		//
		// spriteBatch.Begin(SpriteSortMode.Deferred, BlendState.NonPremultiplied, null, null, null, null, Camera.GetTransformation());
		//
		//-------------------------------

		public Matrix GetTransformation()
		{
			_transform =
			   Matrix.CreateTranslation(new Vector3(-_pos.X, -_pos.Y, 0)) *
			   Matrix.CreateRotationZ(Rotation) *
			   Matrix.CreateScale(new Vector3(Zoom, Zoom, 1)) *
			   Matrix.CreateTranslation(new Vector3(_viewportWidth * 0.5f,
				   _viewportHeight * 0.5f, 0));

			return _transform;
		}

		public Vector2 GetMousePos(Vector2 inMousePos)
		{
			Matrix inverse = Matrix.Invert(this.GetTransformation());
			Vector2 mousePos = Vector2.Transform(new Vector2(inMousePos.X, inMousePos.Y), inverse);

			return mousePos;
		}

		public void SetPos(Vector2 inPos)
		{
			_pos = inPos; //* 0.5f/ Zoom;
		}

		public virtual void UpdateKeyboard(GameTime gameTime, KeyboardState cKey, KeyboardState pKey)
		{
			if(cKey.IsKeyDown(Keys.Space) && pKey.IsKeyUp(Keys.Space))
			{
				this.Zoom = 1f;
			}
		}

		public virtual void UpdateMouse(GameTime intime, MouseState cMouse, MouseState pMouse)
		{
			if (cMouse.ScrollWheelValue > pMouse.ScrollWheelValue)
			{
				this.Zoom += _zoomSpeed;
			}
			else if (cMouse.ScrollWheelValue < pMouse.ScrollWheelValue)
			{
				this.Zoom -= _zoomSpeed;
			}

			Vector2 nextMove = this.Pos;
			if (cMouse.Position.X > _viewportWidth - 50)
				nextMove.X += _panSpeed;
			if (cMouse.Position.X < 50)
				nextMove.X -= _panSpeed;
			if (cMouse.Position.Y > _viewportHeight - 50)
				nextMove.Y += _panSpeed;
			if (cMouse.Position.Y < 50)
				nextMove.Y -= _panSpeed;
			this.Pos = nextMove;
		}
	}
}
