﻿using System.Collections.Generic;
using System.Linq;
using CoolKidEngine.Display;
using CoolKidEngine.Menu;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using RitualisticCapitalism.AssetsManagement;

namespace RitualisticCapitalism
{
	public enum GameStates
	{
		MainMenu,
		QuickGame,
		CustomGame,
		GamePlay
	}
	
	/// <summary>
	/// This is the main type for your game.
	/// </summary>
	public class Game1 : Game
	{
		GraphicsDeviceManager graphics;
		SpriteBatch spriteBatch;

		Menu MainMenu = null;
		GameStates GameState = GameStates.MainMenu;
		Point ScreenSize = new Point(1280, 720);

		MouseState pMouseState;
		KeyboardState pKeyState;

		Hexagon testagon = null;

		Camera2D Camera = null;

		GameMaster GameController = null;

		public Game1()
		{
			graphics = new GraphicsDeviceManager(this);
			Content.RootDirectory = "Content";
			graphics.PreferredBackBufferWidth = ScreenSize.X;
			graphics.PreferredBackBufferHeight = ScreenSize.Y;
			graphics.ApplyChanges();
			IsMouseVisible = true;
		}

		/// <summary>
		/// Allows the game to perform any initialization it needs to before starting to run.
		/// This is where it can query for any required services and load any non-graphic
		/// related content.  Calling base.Initialize will enumerate through any components
		/// and initialize them as well.
		/// </summary>
		protected override void Initialize()
		{
			pMouseState = Mouse.GetState();
			pKeyState = Keyboard.GetState();
			
			Camera = new Camera2D(GraphicsDevice.Viewport, new Vector2(ScreenSize.X/2, ScreenSize.Y/2), 1f, 1.75f, 0.48f);

			// TODO: Add your initialization logic here
			Texture2D texture = new Texture2D(GraphicsDevice, 1, 1);
			texture.SetData<Color>(new Color[] { Color.White });

			SpriteFont menuFont = Content.Load<SpriteFont>("MenuFont");
			SpriteFont gameFont = Content.Load<SpriteFont>("GameFont");

			Label baseLabel = new Label(texture, new Rectangle(0, 0, 320, 60), Color.CadetBlue,
				menuFont, Color.White, "Default");
			Button baseButton = new Button(texture, new Rectangle(0, 0, 240, 80), Color.Gray,
				menuFont, Color.White, "DefaultButton");

			MainMenu = new Menu(baseButton, baseLabel);
			MainMenu.AddLabel("Ritualistic Capitalism", new Vector2(ScreenSize.X / 2, 80));
			MainMenu.AddButton("Quick Start", new Vector2(ScreenSize.X / 2, 200), () => SwitchState(GameStates.QuickGame));
			MainMenu.AddButton("Custom Game", new Vector2(ScreenSize.X / 2, 320), () => SwitchState(GameStates.CustomGame));

			Label gameLabel = new Label(texture, new Rectangle(0, 0, 180, 40), Color.CadetBlue,
				gameFont, Color.White, "Default");
			Button gameButton = new Button(texture, new Rectangle(0, 0, 180, 40), Color.Gray,
				gameFont, Color.White, "DefaultButton");
			Menu gameMenu = new Menu(gameButton, gameLabel);

			ConfigManager masterConfig = new ConfigManager();
			masterConfig.LoadSettings(@"Content\MasterConfig.txt");

			List<BuildingConfig> buildingConfigList = new List<BuildingConfig>();
			foreach(var buildingConfig in masterConfig.ConfigSettings["Buildings"])
			{
				var newConfig = new ConfigManager();
				newConfig.LoadSettings(buildingConfig);
				buildingConfigList.Add(new BuildingConfig(newConfig));
			}

			List<FacilityConfig> facilityConfigList = new List<FacilityConfig>();
			foreach(var facilityConfig in masterConfig.ConfigSettings["Facilities"])
			{
				var newConfig = new ConfigManager();
				newConfig.LoadSettings(facilityConfig);
				facilityConfigList.Add(new FacilityConfig(newConfig));
			}

			CorperationConfigList = new List<CorperationConfig>();
			foreach(var corperateConfig in masterConfig.ConfigSettings["Corperations"])
			{
				var newConfig = new ConfigManager();
				newConfig.LoadSettings(corperateConfig);
				CorperationConfigList.Add(new CorperationConfig(newConfig));
			}

			List<ItemConfig> itemConfigList = new List<ItemConfig>();
			foreach(var itemConfig in masterConfig.ConfigSettings["Items"])
			{
				var newConfig = new ConfigManager();
				newConfig.LoadSettings(itemConfig);
				itemConfigList.Add(new ItemConfig(newConfig));
			}

			MapConfigList = new List<MapConfig>();
			foreach(var mapConfig in masterConfig.ConfigSettings["Maps"])
			{
				var newConfig = new ConfigManager();
				newConfig.LoadSettings(mapConfig);
				MapConfigList.Add(new MapConfig(newConfig));
			}

			Hexagon interiorHex = new Hexagon(new Texture2D(GraphicsDevice, 50, 50), Color.White);
			InteriorTile interiorTile = new InteriorTile(interiorHex.GetTexture(), new Rectangle(0, 0, 50, 50), Color.White);

			GameController = new GameMaster
				(
					buildingConfigList, facilityConfigList, itemConfigList,
					interiorTile,
					Camera, MainMenu, gameMenu, ScreenSize
				);

			base.Initialize();
		}

		protected void SwitchState(GameStates newState)
		{
			switch(newState)
			{
				case GameStates.MainMenu:
					//clean up previous state (probably gameplay)
					break;
				case GameStates.CustomGame:
					//go to custom game menu
					//for now, just have it behave like quick game
					break;
				case GameStates.QuickGame:
					//setup default settings
					//then swith state to gameplay
					GameOptions defaultOptions = new GameOptions()
					{
						StartingMoney = 10000000, //10mil
						NumberOfAIs = 3,
						AITypes = AITypes.Random,
						GamePace = GamePace.Normal
					};
					var city = GameController.GetBuilding("City");
					var gameTileMap = GetTileMap("Mars", city);
					var humanPlayer = new Player(CorperationConfigList.First());
					var players = GetPlayers(humanPlayer, defaultOptions, GraphicsDevice, 25);

					GameController.Setup(players, gameTileMap);
					GameState = GameStates.GamePlay;
					break;
			}
		}

		List<MapConfig> MapConfigList;
        private TileMap GetTileMap(string mapName, Building city)
		{
			var targetMap = MapConfigList.First(x => x.Name.ToLower() == mapName.ToLower());
			int hexSize = 50;
			int mapSize = targetMap.MapSize * hexSize;
			testagon = new Hexagon(new Texture2D(GraphicsDevice, hexSize, hexSize), Color.White);
			LandTile baseHexTile = new LandTile(testagon.GetTexture(), new Rectangle(0, 0, 50, 50), Color.White);
			Camera.SetCameraBoundry(new Rectangle(0, 0, mapSize + 100, mapSize + 100));
			var tileMap = new TileMap(baseHexTile, new Rectangle(100, 100, mapSize, mapSize));

			Player govenment = new Player(null);
			foreach (var citySpot in targetMap.CityLocations)
			{
				var tile = (LandTile)tileMap.GetTileAtPosition(new Vector2(citySpot.Item1 * hexSize, citySpot.Item2 * hexSize));
				tile.SetBuilding((Building)city.Clone());
			}
			return tileMap;
		}

		List<CorperationConfig> CorperationConfigList;
		private List<Player> GetPlayers(Player humanPlayer, GameOptions options, GraphicsDevice device, int logoSize)
		{
			var results = new List<Player>();
			results.Add(humanPlayer);
			humanPlayer.Money = options.StartingMoney;
			humanPlayer.MoneyHistory.Add(options.StartingMoney);

			var usedCorperations = new List<CorperationConfig>();
			usedCorperations.Add(humanPlayer.CorperateConfig);
			for (int x = 0; x < options.NumberOfAIs; x++)
			{
				var corperation = CorperationConfigList.Where(y => !usedCorperations.Contains(y))
					.First();
				Player newPlayer = new Player(corperation);
				newPlayer.IsAI = true;
				newPlayer.Money = options.StartingMoney;
				newPlayer.MoneyHistory.Add(options.StartingMoney);
				results.Add(newPlayer);
				usedCorperations.Add(corperation);
			}

			foreach(var player in results)
			{
				player.Load(device, logoSize);
			}
			return results;
		}

		/// <summary>
		/// LoadContent will be called once per game and is the place to load
		/// all of your content.
		/// </summary>
		protected override void LoadContent()
		{
			// Create a new SpriteBatch, which can be used to draw textures.
			spriteBatch = new SpriteBatch(GraphicsDevice);

			// TODO: use this.Content to load your game content here
			GameController.Load(GraphicsDevice, 50, 50, 25, 25);
		}

		/// <summary>
		/// UnloadContent will be called once per game and is the place to unload
		/// game-specific content.
		/// </summary>
		protected override void UnloadContent()
		{
			// TODO: Unload any non ContentManager content here
		}

		/// <summary>
		/// Allows the game to run logic such as updating the world,
		/// checking for collisions, gathering input, and playing audio.
		/// </summary>
		/// <param name="gameTime">Provides a snapshot of timing values.</param>
		protected override void Update(GameTime gameTime)
		{
			if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed || Keyboard.GetState().IsKeyDown(Keys.Escape))
				Exit();

			MouseState cMouseState = Mouse.GetState();
			KeyboardState cKeyState = Keyboard.GetState();

			// TODO: Add your update logic here
			switch(GameState)
			{
				case GameStates.MainMenu:
					{
						MainMenu.ClickButtons(pMouseState, cMouseState);
					}
                    break;
				case GameStates.GamePlay:
					{
						Camera.UpdateMouse(gameTime, cMouseState, pMouseState);
						Camera.UpdateKeyboard(gameTime, cKeyState, pKeyState);
						GameController.UpdateMouse(cMouseState, pMouseState);
						GameController.Update(gameTime);
					}
					break;
			}

			pMouseState = cMouseState;
			pKeyState = cKeyState;

			base.Update(gameTime);
		}

		/// <summary>
		/// This is called when the game should draw itself.
		/// </summary>
		/// <param name="gameTime">Provides a snapshot of timing values.</param>
		protected override void Draw(GameTime gameTime)
		{
			GraphicsDevice.Clear(Color.Black);

			// TODO: Add your drawing code here
			
			switch(GameState)
			{
				case GameStates.MainMenu:
					spriteBatch.Begin();
					MainMenu.Draw(spriteBatch, gameTime);
					spriteBatch.End();
					break;
				case GameStates.GamePlay:
					GameController.Draw(spriteBatch, gameTime);
					break;
			}
			

			base.Draw(gameTime);
		}
	}
}
