﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using CoolKidEngine.Utility;
using Microsoft.Xna.Framework.Graphics;

namespace RitualisticCapitalism
{
	public class Hexagon
	{
		public Vector2 Center { get; set; }
		public float Radius = 50;
		private float NumPoints = 6;

		float pointSize = 1;

		Texture2D m_Texture = null;
		Color m_Color = Color.White;

		Rectangle m_SourceRect;

		public double Angle
		{
			get { return 2 * MathHelper.Pi / NumPoints; }
		}

		public Hexagon(Texture2D texture, Color color)
		{
			m_Texture = texture;
			m_Color = color;
			m_SourceRect = new Rectangle(0, 0, m_Texture.Width, m_Texture.Height);

			Initialize(m_SourceRect);
		}

		public Texture2D GetTexture()
		{
			return m_Texture;
		}

		protected void Initialize(Rectangle boundingBox)
		{
			Center = new Vector2(boundingBox.Width / 2f, boundingBox.Height / 2f);
			Radius = boundingBox.Width/2;

			Color[] textureColor = new Color[boundingBox.Width * boundingBox.Width];
			//m_Texture.GetData<Color>(textureColor);

			List<Vector2> points = new List<Vector2>();
			for (int i = 0; i < NumPoints; i++)
			{
				float x = Center.X + Radius * (float)Math.Sin(i * Angle);
				float y = Center.Y + Radius * (float)Math.Cos(i * Angle);

				points.Add(new Vector2(x, y));
			}

			for (int i = 0; i < NumPoints; i++)
			{
				Vector2 prevPoint = Vector2.Zero;
				if (i == 0)
				{
					prevPoint = points[(int)NumPoints - 1];
				}
				else
				{
					prevPoint = points[i - 1];
				}

				var distance = CoolMath.GetPointDistance(points[i], prevPoint);
				var dif = points[i] - prevPoint;

				int pointsInLine = (int)(distance / pointSize);

				//sb.Draw(m_Texture, new Vector2(points[i].X, points[i].Y), new Rectangle(0, 0, (int)pointSize, (int)pointSize), m_Color);
				int cleanX = (int)Math.Round(points[i].X, 0, MidpointRounding.AwayFromZero);
				int cleanY = (int)Math.Round(points[i].Y - 0.5f, 0, MidpointRounding.AwayFromZero);

				textureColor[(cleanX * boundingBox.Width + cleanY)] = Color.White;

				for (int w = 1; w <= pointsInLine; w++)
				{
					var thisPoint = points[i] - (dif / pointsInLine * w);
					//sb.Draw(m_Texture, thisPoint, m_Color);
					int cleanPointX = (int)Math.Round(thisPoint.X, 0, MidpointRounding.AwayFromZero);
					int cleanPointY = (int)Math.Round(thisPoint.Y - 0.5f, 0, MidpointRounding.AwayFromZero);
					textureColor[(cleanPointX * boundingBox.Width + cleanPointY)] = Color.White;
				}
			}
			m_Texture.SetData(textureColor);
		}

		public void Draw(SpriteBatch sb, Vector2 postion)
		{
			sb.Draw(m_Texture, postion, m_SourceRect, Color.White);
			
			//List<Vector2> points = new List<Vector2>();
			//for (int i = 0; i < NumPoints; i++)
			//{
			//	float x = Center.X + Radius * (float)Math.Sin(i * Angle);
			//	float y = Center.Y + Radius * (float)Math.Cos(i * Angle);
			//	//Draws the points
			//	//sb.Draw(m_Texture, new Vector2(x, y), new Rectangle(0, 0, 5, 5), m_Color);

			//	points.Add(new Vector2(x, y));
			//}

			//for (int i = 0; i < NumPoints; i++)
			//{
			//	Vector2 prevPoint = Vector2.Zero;
			//	if (i == 0)
			//	{
			//		prevPoint = points[(int)NumPoints - 1];
			//	}
			//	else
			//	{
			//		prevPoint = points[i - 1];
			//	}

			//	var distance = CoolMath.GetPointDistance(points[i], prevPoint);
			//	var dif = points[i] - prevPoint;

			//	int pointsInLine = (int)(distance / pointSize);

			//	sb.Draw(m_Texture, new Vector2(points[i].X, points[i].Y), new Rectangle(0, 0, (int)pointSize, (int)pointSize), m_Color);

			//	for (int w = 1; w <= pointsInLine; w++)
			//	{
			//		var thisPoint = points[i] - (dif / pointsInLine * w);
			//		sb.Draw(m_Texture, thisPoint, m_Color);
			//	}
			//}
		}
	}
}
