﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using CoolKidEngine.Display;
using CoolKidEngine.Utility;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace RitualisticCapitalism
{
	public class TileMap
	{
		public List<Tile> m_TileList { get; protected set; }
		Rectangle m_MapBox { get; set; }

		public TileMap(Tile baseTile, Rectangle boundingBox)
		{
			m_MapBox = boundingBox;

			Initialize(baseTile);
		}

		protected virtual void Initialize(Tile baseTile)
		{
			int tilesPerRow = m_MapBox.Width / baseTile.DestRectangle.Width;
			int tilesPerColumn = m_MapBox.Height / baseTile.DestRectangle.Height;
			Tile[] tileArray = new Tile[tilesPerRow * tilesPerColumn];

			Vector2 startPos = new Vector2(m_MapBox.X, m_MapBox.Y);
			int tileWidth = baseTile.DestRectangle.Width;

			for (int row = 0; row < tilesPerRow; row++)
			{
				for(int column = 0; column < tilesPerColumn; column++)
				{
					var newTile = (Tile)baseTile.Clone();
					var normalVariation = new Vector2((float)(Math.Sqrt(3d)/2d * tileWidth) * column, (tileWidth) * row);
					bool isEven = column % 2 == 0;
					if (isEven)
						newTile.Center = startPos + normalVariation + new Vector2(0, 0);
					else
						newTile.Center = startPos + normalVariation - new Vector2(0, tileWidth/2);
					tileArray[row * tilesPerRow + column] = newTile;
				}
			}

			m_TileList = tileArray.ToList();
		}

		public void Draw(SpriteBatch sb, GameTime time)
		{
			foreach(var tile in m_TileList.Where(x=>x.IsActive))
			{
				tile.Draw(sb, time);
			}
		}

		public Tile GetTileAtPosition(Vector2 position)
		{
			var closest = m_TileList.OrderBy(x => CoolMath.GetPointDistance(x.Center, position)).First();
			return closest;
		}


	}
}
