﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using CoolKidEngine.Display;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using RitualisticCapitalism.AssetsManagement;

namespace RitualisticCapitalism
{
	public class GameTileObject
	{
		protected Tile Image = null;

		public Vector2 Position
		{
			get
			{
				if (Image == null)
					return Vector2.Zero;
				return Image.Position;
			}
			set
			{
				if (Image == null)
					return;
				Image.Position = value;
			}
		}

		public Vector2 Center
		{
			get
			{
				if (Image == null)
					return Vector2.Zero;
				return Image.Center;
			}
			set
			{
				if (Image == null)
					return;
				Image.Center = value;
			}
		}

		public AssetConfig SourceConfig { get; protected set; }
		public GameTileObject(AssetConfig config)
		{
			SourceConfig = config;
		}

		public void Load(GraphicsDevice graphics, int imageSize)
		{
			using (FileStream steam = new FileStream(SourceConfig.FinalImagePath, FileMode.Open))
			{
				Texture2D texture = Texture2D.FromStream(graphics, steam);
				Image = new Tile(texture, new Rectangle(0, 0, imageSize, imageSize), Color.White);
			}
		}

		public virtual void Draw(SpriteBatch sb, GameTime time)
		{
			if (Image != null)
				Image.Draw(sb, time);
		}

		public virtual object CloneInternal(object obj)
		{
			var tileObj = (GameTileObject)obj;
			tileObj.Image = (Tile)Image.Clone();
			return tileObj;
		}

		public virtual object Clone()
		{
			//Config not cloned, it shouldn't be modified anyways
			return CloneInternal(new GameTileObject(SourceConfig));
		}
	}
}
